package at.bestsolution.rsd.rt.client.rest.internal;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import org.eclipse.microprofile.config.ConfigProvider;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import at.bestsolution.rsd.rt.client.ServiceLookup;
import at.bestsolution.rsd.rt.client.rest.RestServiceDescription;
import at.bestsolution.rsd.rt.client.rest.RestServiceFactory;

@Component(property = ServiceLookup.SERVICE_TYPE + ":String=rest")
public class RestServiceLookup implements ServiceLookup {
	private final Map<Class<?>, String> map = new HashMap<>();
	private final Map<Class<?>, RestServiceFactory> factoryMap = new HashMap<>();
	private final Map<Class<?>, Object> serviceCache = new HashMap<>();
	
	private final String host;
	
	public RestServiceLookup() {
		this.host = ConfigProvider.getConfig().getOptionalValue("rsd.rest.url", String.class).orElse("http://localhost:8080");
	}

	@SuppressWarnings("unchecked")
	@Override
	public <S> Optional<S> lookupService(Class<S> clazz) {
		return Optional.ofNullable((S)serviceCache.computeIfAbsent(clazz, this::createService));
	}
	
	private <S> S createService(Class<S> clazz) {
		RestServiceFactory factory = factoryMap.get(clazz);
		String endpoint = map.get(clazz);
		
		if( factory != null && endpoint != null ) {
			return factory.createService(clazz, this.host + "/" + endpoint);
		}
		
		return null;
	}

	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void registerRegisterService(RestServiceDescription restService) {
		synchronized (map) {
			map.put(restService.type(), restService.endpoint());
		}
	}

	public void unregisterRegisterService(RestServiceDescription restService) {
		synchronized (map) {
			map.remove(restService.type());
		}
	}

	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void registerRegisterFactory(RestServiceFactory factory) {
		synchronized (factoryMap) {
			for (Class<?> cl : factory.serviceTypes()) {
				factoryMap.put(cl, factory);
			}
		}
	}

	public void unregisterRegisterFactory(RestServiceFactory factory) {
		synchronized (factoryMap) {
			for (Class<?> cl : factory.serviceTypes()) {
				factoryMap.remove(cl);
			}
		}
	}
}
